@@ -6,6 +6,7 @@ import websockets
 import ssl
 import logging
 import time
+from urllib.parse import urlparse, urlunparse
 from jupyter_server.base.handlers import JupyterHandler
 from jupyter_server.utils import url_path_join
 import tornado
@@ -153,6 +154,61 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
     WebSocket proxy handler that forwards connections from /ws to target server
     """
     
+    def _resolve_target_url_for_docker(self, url):
+        """
+        Resolve target URL for Docker environment.
+        Replace localhost/127.0.0.1 with Docker host IP when running in container.
+        """
+        # Enhanced Docker detection with multiple methods
+        docker_indicators = []
+        
+        # Method 1: Check for /.dockerenv file
+        dockerenv_exists = os.path.exists('/.dockerenv')
+        docker_indicators.append(f"/.dockerenv exists: {dockerenv_exists}")
+        
+        # Method 2: Check environment variable
+        docker_env = os.getenv('DOCKER_CONTAINER') == 'true'
+        docker_indicators.append(f"DOCKER_CONTAINER env: {docker_env}")
+        
+        # Method 3: Check /proc/1/cgroup for docker
+        cgroup_docker = False
+        try:
+            if os.path.exists('/proc/1/cgroup'):
+                with open('/proc/1/cgroup', 'r') as f:
+                    cgroup_content = f.read()
+                    cgroup_docker = 'docker' in cgroup_content or 'containerd' in cgroup_content
+                docker_indicators.append(f"/proc/1/cgroup contains docker/containerd: {cgroup_docker}")
+        except Exception as e:
+            docker_indicators.append(f"/proc/1/cgroup check failed: {e}")
+        
+        # Determine if we're in Docker
+        is_docker = dockerenv_exists or docker_env or cgroup_docker
+        
+        print(f"[ESCOBAR-WS] Docker detection indicators: {docker_indicators}")
+        print(f"[ESCOBAR-WS] Final Docker detection result: {is_docker}")
+        
+        if not is_docker:
+            print(f"[ESCOBAR-WS] Not in Docker container, using original URL: {url}")
+            return url
+        
+        # Parse the URL to extract components
+        parsed = urlparse(url)
+        print(f"[ESCOBAR-WS] Parsed URL - hostname: '{parsed.hostname}', netloc: '{parsed.netloc}'")
+        
+        # Check if hostname is localhost or 127.0.0.1
+        if parsed.hostname in ['localhost', '127.0.0.1']:
+            # Replace with Docker host IP
+            new_netloc = parsed.netloc.replace(parsed.hostname, '172.17.0.1')
+            new_parsed = parsed._replace(netloc=new_netloc)
+            new_url = urlunparse(new_parsed)
+            
+            print(f"[ESCOBAR-WS] Docker hostname resolution: {url} → {new_url}")
+            return new_url
+        else:
+            print(f"[ESCOBAR-WS] Docker container detected, but hostname '{parsed.hostname}' is not localhost/127.0.0.1, keeping original: {url}")
+        
+        return url
+    
     def __init__(self, *args, **kwargs):
         print(f"[ESCOBAR-WS] WebSocketProxyHandler.__init__ called")
         super().__init__(*args, **kwargs)
@@ -166,15 +222,67 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
         except:
             print(f"[ESCOBAR-WS] Could not get hostname")
         
-        # Read target URL from environment variable
-        self.target_url = os.getenv('WEBSOCKET_PROXY_TARGET', 'ws://localhost:8777/ws')
-        print(f"[ESCOBAR-WS] Target URL will be: {self.target_url}")
+        # Store raw target URL from environment variable (resolve per-connection)
+        self.raw_target_url = os.getenv('WEBSOCKET_PROXY_TARGET', 'ws://localhost:8777/ws')
+        print(f"[ESCOBAR-WS] Raw target URL stored: {self.raw_target_url}")
+        print(f"[ESCOBAR-WS] Docker resolution will be applied per-connection")
         
         # Debug all WebSocket-related environment variables
         websocket_env_vars = [(k, v) for k, v in os.environ.items() if 'WEBSOCKET' in k.upper()]
         print(f"[ESCOBAR-WS] All WEBSOCKET environment vars: {websocket_env_vars}")
         
         self.is_closing = False
+    
+    def _get_user_bonnie_url(self):
+        """
+        Get user-configured Bonnie URL from bonnie_url query parameter.
+        This allows the frontend to override the environment variable.
+        """
+        print(f"[ESCOBAR-WS] === READING USER BONNIE URL FROM QUERY PARAMETER ===")
+        
+        try:
+            # Get the bonnieUrl from query parameters
+            bonnie_url = self.get_argument('bonnie_url', None)
+            print(f"[ESCOBAR-WS] bonnie_url query parameter value: '{bonnie_url}'")
+            
+            if bonnie_url and bonnie_url.strip():
+                # Validate that it's a WebSocket URL
+                if bonnie_url.startswith(('ws://', 'wss://')):
+                    print(f"[ESCOBAR-WS] Valid user Bonnie URL from query parameter: {bonnie_url}")
+                    return bonnie_url.strip()
+                else:
+                    print(f"[ESCOBAR-WS] Invalid Bonnie URL format (must start with ws:// or wss://): {bonnie_url}")
+                    return None
+            else:
+                print(f"[ESCOBAR-WS] No bonnie_url query parameter provided")
+                return None
+                
+        except Exception as e:
+            print(f"[ESCOBAR-WS] Error reading bonnie_url query parameter: {e}")
+            return None
+        finally:
+            print(f"[ESCOBAR-WS] === END BONNIE URL QUERY PARAMETER READ ===")
+    
+    def _get_target_url(self):
+        """
+        Get the target URL with priority: User Setting > Environment Variable > Default
+        """
+        # Priority 1: User-configured Bonnie URL
+        user_bonnie_url = self._get_user_bonnie_url()
+        if user_bonnie_url:
+            print(f"[ESCOBAR-WS] Using user-configured Bonnie URL: {user_bonnie_url}")
+            return user_bonnie_url
+        
+        # Priority 2: Environment variable
+        env_target_url = os.getenv('WEBSOCKET_PROXY_TARGET')
+        if env_target_url:
+            print(f"[ESCOBAR-WS] Using environment WEBSOCKET_PROXY_TARGET: {env_target_url}")
+            return env_target_url
+        
+        # Priority 3: Default fallback
+        default_url = 'ws://localhost:8777/ws'
+        print(f"[ESCOBAR-WS] Using default target URL: {default_url}")
+        return default_url
         
     def check_origin(self, origin):
         """Allow connections from any origin (adjust as needed for security)"""
@@ -184,11 +292,18 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
         """Called when websocket connection is opened"""
         start_time = time.time()
         
+        # Get target URL with priority: User Setting > Environment Variable > Default
+        raw_target_url = self._get_target_url()
+        
+        # Apply Docker hostname resolution for this connection
+        target_url = self._resolve_target_url_for_docker(raw_target_url)
+        
         # Log which endpoint was accessed and path normalization
         request_path = self.request.path
         print(f"[ESCOBAR-WS] === CLIENT CONNECTION OPENED ===")
         print(f"[ESCOBAR-WS] Client connected via: {request_path}")
-        print(f"[ESCOBAR-WS] Target remains: {self.target_url}")
+        print(f"[ESCOBAR-WS] Raw target URL: {raw_target_url}")
+        print(f"[ESCOBAR-WS] Resolved target URL for this connection: {target_url}")
         if request_path != "/ws":
             print(f"[ESCOBAR-WS] Path normalization: {request_path} → /ws")
         
@@ -196,9 +311,9 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
         print(f"[ESCOBAR-WS] Client origin: {self.request.headers.get('Origin', 'NO_ORIGIN')}")
         print(f"[ESCOBAR-WS] Client remote IP: {self.request.remote_ip}")
         print(f"[ESCOBAR-WS] Request headers: {dict(self.request.headers)}")
-        print(f"[ESCOBAR-WS] Attempting to connect to target: {self.target_url}")
+        print(f"[ESCOBAR-WS] Attempting to connect to target: {target_url}")
         
-        logging.info(f"WebSocket connection opened, proxying to {self.target_url}")
+        logging.info(f"WebSocket connection opened, proxying to {target_url}")
         
         try:
             # Establish connection to target websocket server
@@ -216,19 +331,20 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
             print(f"[ESCOBAR-WS] Headers to forward: {headers}")
             
             # Determine if we need SSL based on URL scheme
-            use_ssl = self.target_url.startswith('wss://')
+            use_ssl = target_url.startswith('wss://')
             ssl_context = ssl.create_default_context() if use_ssl else None
             print(f"[ESCOBAR-WS] Using SSL: {use_ssl}")
             
-            print(f"[ESCOBAR-WS] Attempting websockets.connect() to {self.target_url}")
+            print(f"[ESCOBAR-WS] Attempting websockets.connect() to {target_url}")
             
             # Connect to target websocket (works with both ws:// and wss://)
             self.target_ws = await websockets.connect(
-                self.target_url,
+                target_url,
                 additional_headers=headers,
                 ssl=ssl_context,
                 ping_interval=20,
-                ping_timeout=10
+                ping_timeout=10,
+                max_size=100 * 1024 * 1024 
             )
             
             end_time = time.time()
@@ -239,7 +355,7 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
             asyncio.create_task(self._forward_from_target())
             
             print(f"[ESCOBAR-WS] === CONNECTION SETUP COMPLETE ===")
-            logging.info(f"Successfully connected to target websocket server: {self.target_url}")
+            logging.info(f"Successfully connected to target websocket server: {target_url}")
             
         except Exception as e:
             end_time = time.time()
@@ -247,15 +363,15 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
             print(f"[ESCOBAR-WS] Connection failed after {end_time - start_time:.2f} seconds")
             print(f"[ESCOBAR-WS] Error: {str(e)}")
             print(f"[ESCOBAR-WS] Error type: {type(e).__name__}")
-            print(f"[ESCOBAR-WS] Target URL: {self.target_url}")
+            print(f"[ESCOBAR-WS] Target URL: {target_url}")
             if hasattr(e, 'errno'):
                 print(f"[ESCOBAR-WS] Errno: {e.errno}")
             if hasattr(e, 'strerror'):
                 print(f"[ESCOBAR-WS] Strerror: {e.strerror}")
             print(f"[ESCOBAR-WS] === END CONNECTION FAILED ===")
             
-            logging.error(f"Failed to connect to target websocket {self.target_url}: {str(e)}")
-            self.close(code=1011, reason=f"Failed to connect to target server: {self.target_url}")
+            logging.error(f"Failed to connect to target websocket {target_url}: {str(e)}")
+            self.close(code=1011, reason=f"Failed to connect to target server: {target_url}")
     
     async def on_message(self, message):
         """Called when a message is received from the client"""
@@ -309,16 +425,36 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
                     break
                 print(f"[ESCOBAR-WS] === END TARGET MESSAGE #{message_count} ===")
                 
-        except websockets.exceptions.ConnectionClosed:
-            print(f"[ESCOBAR-WS] Target websocket connection closed normally")
-            logging.info("Target websocket connection closed")
+        except websockets.exceptions.ConnectionClosed as e:
+            print(f"[ESCOBAR-WS] === TARGET CONNECTION CLOSED DETAILS ===")
+            print(f"[ESCOBAR-WS] Target websocket connection closed")
+            print(f"[ESCOBAR-WS] Close code: {getattr(e, 'code', 'NO_CODE')}")
+            print(f"[ESCOBAR-WS] Close reason: {getattr(e, 'reason', 'NO_REASON')}")
+            print(f"[ESCOBAR-WS] Exception type: {type(e).__name__}")
+            print(f"[ESCOBAR-WS] Exception str: {str(e)}")
+            print(f"[ESCOBAR-WS] Target WS final state: {getattr(self.target_ws, 'state', 'NO_STATE') if self.target_ws else 'NO_TARGET_WS'}")
+            print(f"[ESCOBAR-WS] Messages processed before disconnect: {message_count}")
+            print(f"[ESCOBAR-WS] === END TARGET CONNECTION CLOSED DETAILS ===")
+            
+            logging.info(f"Target websocket connection closed - code: {getattr(e, 'code', 'NO_CODE')}, reason: {getattr(e, 'reason', 'NO_REASON')}")
+            
             if not self.is_closing:
                 print(f"[ESCOBAR-WS] Closing client connection due to target disconnect")
                 self.close(code=1011, reason="Target server disconnected")
         except Exception as e:
-            print(f"[ESCOBAR-WS] ERROR in target message forwarding:")
-            print(f"[ESCOBAR-WS]   Error: {str(e)}")
-            print(f"[ESCOBAR-WS]   Error type: {type(e).__name__}")
+            print(f"[ESCOBAR-WS] === TARGET MESSAGE FORWARDING ERROR ===")
+            print(f"[ESCOBAR-WS] Error: {str(e)}")
+            print(f"[ESCOBAR-WS] Error type: {type(e).__name__}")
+            print(f"[ESCOBAR-WS] Messages processed before error: {message_count}")
+            print(f"[ESCOBAR-WS] Target WS state: {getattr(self.target_ws, 'state', 'NO_STATE') if self.target_ws else 'NO_TARGET_WS'}")
+            if hasattr(e, '__dict__'):
+                print(f"[ESCOBAR-WS] Error attributes: {e.__dict__}")
+            if hasattr(e, 'errno'):
+                print(f"[ESCOBAR-WS] Errno: {e.errno}")
+            if hasattr(e, 'strerror'):
+                print(f"[ESCOBAR-WS] Strerror: {e.strerror}")
+            print(f"[ESCOBAR-WS] === END TARGET MESSAGE FORWARDING ERROR ===")
+            
             logging.error(f"Error receiving from target websocket: {str(e)}")
             if not self.is_closing:
                 print(f"[ESCOBAR-WS] Closing client connection due to target error")
@@ -348,17 +484,36 @@ class WebSocketProxyHandler(tornado.websocket.WebSocketHandler):
         """Safely close the target websocket connection"""
         print(f"[ESCOBAR-WS] === CLOSING TARGET CONNECTION ===")
         try:
-            if self.target_ws and not self.target_ws.closed:
-                print(f"[ESCOBAR-WS] Target WS is open, closing it")
-                await self.target_ws.close()
-                print(f"[ESCOBAR-WS] Target connection closed successfully")
-                logging.info("Target websocket connection closed")
+            if self.target_ws:
+                print(f"[ESCOBAR-WS] Target WS exists, checking state")
+                print(f"[ESCOBAR-WS] Target WS type: {type(self.target_ws)}")
+                print(f"[ESCOBAR-WS] Target WS state: {getattr(self.target_ws, 'state', 'NO_STATE_ATTR')}")
+                
+                # Check if connection is already closed using the correct websockets library API
+                try:
+                    is_closed = self.target_ws.state.name == 'CLOSED'
+                    print(f"[ESCOBAR-WS] Target WS is closed: {is_closed}")
+                except AttributeError:
+                    # Fallback: just try to close it regardless of state
+                    print(f"[ESCOBAR-WS] Cannot check state, will attempt to close anyway")
+                    is_closed = False
+                
+                if not is_closed:
+                    print(f"[ESCOBAR-WS] Target WS is open, closing it")
+                    await self.target_ws.close()
+                    print(f"[ESCOBAR-WS] Target connection closed successfully")
+                    logging.info("Target websocket connection closed")
+                else:
+                    print(f"[ESCOBAR-WS] Target WS already closed")
             else:
-                print(f"[ESCOBAR-WS] Target WS already closed or None")
+                print(f"[ESCOBAR-WS] No target WS to close")
         except Exception as e:
             print(f"[ESCOBAR-WS] ERROR closing target connection:")
             print(f"[ESCOBAR-WS]   Error: {str(e)}")
             print(f"[ESCOBAR-WS]   Error type: {type(e).__name__}")
+            print(f"[ESCOBAR-WS]   Target WS type: {type(self.target_ws) if self.target_ws else 'None'}")
+            if hasattr(e, '__dict__'):
+                print(f"[ESCOBAR-WS]   Error attributes: {e.__dict__}")
             logging.error(f"Error closing target websocket: {str(e)}")
         
         print(f"[ESCOBAR-WS] === END CLOSING TARGET CONNECTION ===")
@@ -373,6 +528,47 @@ def setup_handlers(web_app):
     print(f"[ESCOBAR-WS] Host pattern: {host_pattern}")
     print(f"[ESCOBAR-WS] Base URL: {base_url}")
     
+    # Debug web_app settings to find settings manager
+    print(f"[ESCOBAR-WS] Web app settings keys: {list(web_app.settings.keys())}")
+    
+    # Try to find the settings manager
+    settings_manager = None
+    
+    # Method 1: Check if it's directly in web_app.settings
+    if 'settings_manager' in web_app.settings:
+        settings_manager = web_app.settings['settings_manager']
+        print(f"[ESCOBAR-WS] Found settings manager in web_app.settings")
+    
+    # Method 2: Check if there's a serverapp with settings manager
+    elif hasattr(web_app, 'serverapp') and hasattr(web_app.serverapp, 'settings_manager'):
+        settings_manager = web_app.serverapp.settings_manager
+        print(f"[ESCOBAR-WS] Found settings manager in web_app.serverapp")
+    
+    # Method 3: Check if there's a parent app with settings manager
+    elif hasattr(web_app, 'parent') and hasattr(web_app.parent, 'settings_manager'):
+        settings_manager = web_app.parent.settings_manager
+        print(f"[ESCOBAR-WS] Found settings manager in web_app.parent")
+    
+    # Method 4: Try to import and get the settings manager directly
+    else:
+        try:
+            from jupyter_server.services.settings.manager import SettingsManager
+            # Try to create a settings manager instance
+            settings_manager = SettingsManager()
+            print(f"[ESCOBAR-WS] Created new settings manager instance")
+        except Exception as e:
+            print(f"[ESCOBAR-WS] Could not create settings manager: {e}")
+    
+    print(f"[ESCOBAR-WS] Settings manager: {settings_manager}")
+    print(f"[ESCOBAR-WS] Settings manager type: {type(settings_manager) if settings_manager else 'None'}")
+    
+    # Store the settings manager in web_app.settings for handlers to access
+    if settings_manager:
+        web_app.settings['settings_manager'] = settings_manager
+        print(f"[ESCOBAR-WS] Stored settings manager in web_app.settings")
+    else:
+        print(f"[ESCOBAR-WS] WARNING: No settings manager available - user bonnieUrl will not work")
+    
     # Register the /proxy endpoint with a path parameter
     proxy_pattern = url_path_join(base_url, "proxy", "(.*)")
     print(f"[ESCOBAR-WS] Proxy pattern: {proxy_pattern}")
