@@ -37,20 +37,38 @@ const DEFAULT_TIMEOUT = 1000 * 60 * 60;
 /**
  * Get the username for the current user
  * 
+ * @param settingsManager Optional settings manager to get username from settings
  * @returns The username string
  */
-function getUserName(): string {
-  // First check if we're in JupyterHub mode
-  const isJupyterHub = window.location.href.includes('/user/');
+function getUserName(settingsManager?: SettingsManager): string {
+  // Get current settings if settings manager is available
+  let settings: ILocalSettings | null = null;
+  if (settingsManager) {
+    settings = settingsManager.getLocalSettings();
+  }
 
-  if (isJupyterHub) {
-    // In JupyterHub mode, get the username from the URL
+  // Check if we should use JupyterHub username (either from settings or auto-detection)
+  const shouldUseJupyterHub = settings?.usernameFromJupyterHub || window.location.href.includes('/user/');
+
+  if (shouldUseJupyterHub) {
+    // Try to get the username from JupyterHub
     const usernameInfo = getJupyterHubUsername();
     if (usernameInfo.fromJupyterHub) {
       return usernameInfo.username;
     }
   }
 
+  // If we have settings and a configured username, use it
+  if (settings?.username && settings.username !== 'User') {
+    return settings.username;
+  }
+
+  // Fall back to default username from settings or hardcoded default
+  if (settings?.username) {
+    return settings.username;
+  }
+
+  // Final fallback to VoittaDefaultUser (should rarely be reached)
   const hostname = window.location.hostname || 'localhost';
   return `VoittaDefaultUser@${hostname}`;
 }
@@ -334,7 +352,7 @@ export class ChatWidget extends Widget {
     });
 
     // Initialize the message handler with default settings
-    const currentUsername = getUserName();
+    const currentUsername = getUserName(this.settingsManager);
     const defaultSettings = this.settingsManager.getCompleteSettings();
     this.messageHandler = new MessageHandler(
       defaultSettings.voittaApiKey,
@@ -368,6 +386,9 @@ export class ChatWidget extends Widget {
     (window as any).escobarAvailableModels = this.availableModels;
     (window as any).escobarCurrentSettings = defaultSettings;
 
+    // One-time cleanup: Remove any existing localStorage settings
+    this.cleanupLocalStorage();
+
     // Initialize settings and start the application
     setTimeout(async () => {
       await this.initializeSettings();
@@ -462,6 +483,21 @@ export class ChatWidget extends Widget {
     }
   }
 
+  /**
+   * One-time cleanup: Remove any existing localStorage settings
+   */
+  private cleanupLocalStorage(): void {
+    try {
+      const existingSettings = localStorage.getItem('escobar-settings');
+      if (existingSettings) {
+        localStorage.removeItem('escobar-settings');
+        console.log('🧹 CLEANUP: Removed legacy localStorage settings');
+      }
+    } catch (error) {
+      console.warn('🧹 CLEANUP: Failed to clean localStorage:', error);
+    }
+  }
+
   /**
    * Initialize settings using the SettingsManager
    */
@@ -480,7 +516,7 @@ export class ChatWidget extends Widget {
     console.log('🔧 SETTINGS: Settings change detected');
 
     // Update message handler with new settings
-    const currentUsername = getUserName();
+    const currentUsername = getUserName(this.settingsManager);
     this.messageHandler.updateSettings(
       settings.voittaApiKey,
       settings.openaiApiKey,
@@ -495,19 +531,7 @@ export class ChatWidget extends Widget {
     (window as any).escobarCurrentSettings = settings;
     (window as any).escobarUsername = currentUsername;
 
-    // Check if server URL changed and reinitialize if needed
-    const currentSettings = this.settingsManager.getCompleteSettings();
-    if (currentSettings.serverUrl) {
-      // Only reinitialize if this is NOT from a settings save operation
-      // (to avoid the timing issue where we reconnect during save)
-      const isFromSettingsSave = (window as any).escobarSettingsSaveInProgress;
-      if (!isFromSettingsSave) {
-        console.log('🔧 SETTINGS: Server URL changed, reinitializing connection');
-        this.reinitializeConnection();
-      } else {
-        console.log('🔧 SETTINGS: Skipping reconnection during settings save');
-      }
-    }
+    console.log('🔧 SETTINGS: Updated username to:', currentUsername);
   }
 
   /**
@@ -534,11 +558,25 @@ export class ChatWidget extends Widget {
     const tools = await get_tools(this.app, this.notebookTracker, this.debuggerService);
 
     // Get the current username using the new getUserName function
-    const currentUsername = getUserName();
+    const currentUsername = getUserName(this.settingsManager);
+
+    // Update the message handler's username to ensure it's current
+    const currentSettings = this.settingsManager.getCompleteSettings();
+    this.messageHandler.updateSettings(
+      currentSettings.voittaApiKey,
+      currentSettings.openaiApiKey,
+      currentSettings.anthropicApiKey,
+      currentSettings.geminiApiKey,
+      '', // selectedProvider will be determined dynamically
+      currentUsername,
+      currentSettings.maxMessages
+    );
 
     // Update global variable
     (window as any).escobarUsername = currentUsername;
 
+    console.log('🔧 INIT: Updated username to:', currentUsername);
+
     registerFunction('handleResponse', false, this.handlePythonResponse.bind(this));
     registerFunction('say', false, this.say.bind(this));
     registerFunction('tool_say', false, this.tool_say.bind(this));
@@ -591,6 +629,9 @@ export class ChatWidget extends Widget {
         window.dispatchEvent(new CustomEvent('escobar-models-updated', {
           detail: { models: this.availableModels }
         }));
+
+        // Update the model dropdown with the loaded models
+        this.updateModelDropdown();
       } else {
         console.log('📋 MODELS: No models received from server');
         this.availableModels = [];
@@ -814,7 +855,7 @@ export class ChatWidget extends Widget {
    * Refresh the list of available chats
    */
   private async refreshChatList(): Promise<IChatInfo[]> {
-    const currentUsername = getUserName();
+    const currentUsername = getUserName(this.settingsManager);
 
     try {
       const listChatsCallId = this.messageHandler.generateMessageId();
