@@ -53,6 +53,51 @@ export function get_ws() {
   return ws;
 }
 
+/**
+ * Resolve WebSocket URL for JupyterHub environment
+ * @param url - The URL to resolve (can be relative or absolute)
+ * @returns Resolved URL with proper JupyterHub user path if needed
+ */
+function resolveWebSocketURL(url: string): string {
+  console.log(`##>>## Original URL: ${url}`);
+
+  // If it's already an absolute URL (starts with ws:// or wss://), return as-is
+  if (url.startsWith('ws://') || url.startsWith('wss://')) {
+    console.log(`##>>## Absolute URL, using as-is: ${url}`);
+    return url;
+  }
+
+  // Check if we're in JupyterHub environment
+  const currentPath = window.location.pathname;
+  const hubUserMatch = currentPath.match(/^\/user\/([^\/]+)\//);
+
+  if (hubUserMatch) {
+    // In JupyterHub - construct user-specific URL
+    const userPath = hubUserMatch[0]; // "/user/roman.semein@gmail.com/"
+    const resolvedPath = userPath.slice(0, -1) + url; // "/user/roman.semein@gmail.com/ws"
+
+    // Construct full WebSocket URL
+    const protocol = window.location.protocol === 'https:' ? 'wss:' : 'ws:';
+    const host = window.location.host;
+    const fullURL = `${protocol}//${host}${resolvedPath}`;
+
+    console.log(`##>>## JupyterHub detected, resolved URL: ${fullURL}`);
+    console.log(`##>>## User path: ${userPath}`);
+    console.log(`##>>## Resolved path: ${resolvedPath}`);
+
+    return fullURL;
+  } else {
+    // Standalone JupyterLab - construct normal URL
+    const protocol = window.location.protocol === 'https:' ? 'wss:' : 'ws:';
+    const host = window.location.host;
+    const fullURL = `${protocol}//${host}${url}`;
+
+    console.log(`##>>## Standalone JupyterLab, resolved URL: ${fullURL}`);
+
+    return fullURL;
+  }
+}
+
 let stopEventListenerAdded = false;
 let stopRequested = false;
 
@@ -60,7 +105,7 @@ let stopRequested = false;
  * Initializes the WebSocket connection to the Python server.
  * If a connection attempt is already in progress or the connection is active, it returns that promise.
  *
- * @param url - The WebSocket URL (default: "ws://localhost:8000/ws")
+ * @param url - The WebSocket URL (default: "ws://127.0.0.1:8777/ws")
  * @returns A Promise that resolves when the connection is successfully established.
  */
 export function initPythonBridge(url: string = "ws://127.0.0.1:8777/ws"): Promise<void> {
@@ -85,8 +130,25 @@ export function initPythonBridge(url: string = "ws://127.0.0.1:8777/ws"): Promis
   */
 
   connectionPromise = new Promise<void>((resolve, reject) => {
-    console.log(">>> Connecting to " + url);
-    ws = new WebSocket(url);
+    // Always resolve the URL for JupyterHub environment (never connect directly to bonnieUrl)
+    let resolvedURL = resolveWebSocketURL(url);
+
+    // Check if we need to add bonnieUrl as query parameter for backend proxy
+    const currentSettings = (window as any).escobarCurrentSettings;
+    if (currentSettings?.bonnieUrl && url.startsWith('/')) {
+      console.log(`>>> Adding bonnieUrl as proxy target: ${currentSettings.bonnieUrl}`);
+
+      // Add bonnieUrl as query parameter for backend to use as proxy target
+      const separator = resolvedURL.includes('?') ? '&' : '?';
+      resolvedURL += `${separator}bonnie_url=${encodeURIComponent(currentSettings.bonnieUrl)}`;
+
+      console.log(`>>> Final URL with bonnieUrl parameter: ${resolvedURL}`);
+    } else {
+      console.log(">>> No bonnieUrl parameter needed");
+    }
+
+    console.log(">>> Connecting to " + resolvedURL);
+    ws = new WebSocket(resolvedURL);
 
     ws.addEventListener('open', (event) => {
       isConnected = true;
@@ -257,22 +319,13 @@ export async function callPython(message: import('../types/protocol').ProtocolMe
   let user_id = '';
   let session_id = '';
 
-  // First check if it's set as a global variable by the chat widget
+  // Get username from global variable set by the chat widget
   if ((window as any).escobarUsername) {
     user_id = (window as any).escobarUsername;
-  }
-  // If not, try to get it from localStorage
-  else {
-    try {
-      const savedSettings = localStorage.getItem('escobar-settings');
-      if (savedSettings) {
-        const settings = JSON.parse(savedSettings);
-        user_id = settings.username || 'VoittaDefaultUser';
-      }
-    } catch (error) {
-      console.error('Failed to get username from localStorage:', error);
-      user_id = 'VoittaDefaultUser';
-    }
+  } else {
+    // Fallback to default if global variable not available
+    console.warn('Username not available from global variable, using default');
+    user_id = 'VoittaDefaultUser';
   }
 
   // Get session_id from the message if it's a chat-related message
